/**
 * @file bootstrap_toast_messages.js
 *
 * Javascript for integrating the bootstrap toast JS messages into drupal
 * Replaces the standard drupal status messages and creates bootstrap 4 toast instead
 */
(function ($, Drupal, once, drupalSettings) {

  'use strict';

  Drupal.bootstrap_toast_messages = {
    id: 'drupal-bootstrap-toast-messages',
    context: 'body',
    bootstrapVersion: null,
    defaultDuration: 10000,
    animation: {
      in: 'fade-in',
      out: 'fade-out',
      duration: 300,

    },
    messageTypeDuration: {
      status: null,
      success: null,
      warning: null,
    },
    dataAttributes: {
      dismiss: 'data-bs-dismiss',
      delay: 'data-bs-delay',
      autoHide: 'data-bs-autohide',
      animation: 'data-bs-animation',
    },
    position: 'top-right', /** top-left/top-right/top-center/bottom-left/bottom-right/bottom-center - Where the toast will show up **/
    pauseDelayOnHover: true, /** true/false - If you want to pause the delay of toast when hovering over the toast **/
    defaultMessageType: 'status',
    defaultMessageTitle: 'Info',
    iconInfo: '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-info-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14m0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16"/><path d="m8.93 6.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533zM9 4.5a1 1 0 1 1-2 0 1 1 0 0 1 2 0"/></svg>',
    iconStatus: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="#4ca154" class="bi bi-check2-circle mr-2" viewBox="0 0 16 16"><path d="M2.5 8a5.5 5.5 0 0 1 8.25-4.764.5.5 0 0 0 .5-.866A6.5 6.5 0 1 0 14.5 8a.5.5 0 0 0-1 0 5.5 5.5 0 1 1-11 0"/><path d="M15.354 3.354a.5.5 0 0 0-.708-.708L8 9.293 5.354 6.646a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0z"/></svg>',
    iconWarning: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="#cc7c2e" class="bi bi-exclamation-triangle mr-2" viewBox="0 0 16 16"><path d="M7.938 2.016A.13.13 0 0 1 8.002 2a.13.13 0 0 1 .063.016.15.15 0 0 1 .054.057l6.857 11.667c.036.06.035.124.002.183a.2.2 0 0 1-.054.06.1.1 0 0 1-.066.017H1.146a.1.1 0 0 1-.066-.017.2.2 0 0 1-.054-.06.18.18 0 0 1 .002-.183L7.884 2.073a.15.15 0 0 1 .054-.057m1.044-.45a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767z"/><path d="M7.002 12a1 1 0 1 1 2 0 1 1 0 0 1-2 0M7.1 5.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0z"/></svg>',
    iconError: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="#ca3a31" class="bi bi-exclamation-octagon mr-2" viewBox="0 0 16 16"><path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0M7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0z"/></svg>',
    container: '<div id="toast-container" class="toast-container" aria-live="polite" aria-atomic="true"></div>',
    messages: [],
    count: 0,
    totalRemovingHeight: 0,
    init: function () {
      this.detectVersion();
      this.getSettings();
      this.setAttributes();
    },
    detectVersion: function () {
      if (typeof bootstrap === 'undefined' || typeof bootstrap.Toast === 'undefined') {
        console.error('Bootstrap or bootstrap.Toast is not loaded.');
        return;
      }
      if (bootstrap.Toast.VERSION.startsWith(4)) {
        this.bootstrapVersion = 4;
      } else if (bootstrap.Toast.VERSION.startsWith(5)) {
        this.bootstrapVersion = 5;
      } else {
        console.error('Bootstrap Toast version ' + bootstrap.Toast.VERSION + ' is not supported.');
      }
      //console.log('Init() Bootstrap Toast Version', bootstrap.Toast.VERSION);
    },
    getSettings: function () {
      if (typeof drupalSettings.bootstrap_toast_messages !== 'undefined') {
        if (typeof drupalSettings.bootstrap_toast_messages.status_duration !== 'undefined') {
          this.messageTypeDuration.status = drupalSettings.bootstrap_toast_messages.status_duration;
        } else {
          this.messageTypeDuration.status = this.defaultDuration;
        }
        if (typeof drupalSettings.bootstrap_toast_messages.warning_duration !== 'undefined') {
          this.messageTypeDuration.warning = drupalSettings.bootstrap_toast_messages.warning_duration;
        } else {
          this.messageTypeDuration.warning = this.defaultDuration;
        }
        if (typeof drupalSettings.bootstrap_toast_messages.error_duration !== 'undefined') {
          this.messageTypeDuration.error = drupalSettings.bootstrap_toast_messages.error_duration;
        } else {
          this.messageTypeDuration.error = this.defaultDuration;
        }
        if (typeof drupalSettings.bootstrap_toast_messages.position !== 'undefined') {
          this.position = drupalSettings.bootstrap_toast_messages.position;
        }
        if (typeof drupalSettings.bootstrap_toast_messages.animation_in !== 'undefined') {
          this.animation.in = drupalSettings.bootstrap_toast_messages.animation_in;
        }
        if (typeof drupalSettings.bootstrap_toast_messages.animation_out !== 'undefined') {
          this.animation.out = drupalSettings.bootstrap_toast_messages.animation_out;
        }
      }
    },
    setAttributes: function () {
      if (this.bootstrapVersion === 4) {
        this.dataAttributes.dismiss = 'data-dismiss';
        this.dataAttributes.delay = 'data-delay';
        this.dataAttributes.autoHide = 'data-autohide';
        this.dataAttributes.animation = 'data-animation';
      }
    },
    createMessages: function (messages_array) {
      if (!this.bootstrapVersion) {
        console.error('Bootstrap version not detected.');
        return;
      }
      //this.removeToasts();
      this.messages = messages_array;
      if (Array.isArray(this.messages) && this.messages.length) {
        for (let i in this.messages) {
          switch (this.messages[i].type) {
            case 'status':
            case 'success':
              this.messages[i].type = 'success';
              this.messages[i].drupalType = 'status';
              this.messages[i].icon = this.iconStatus;
              this.messages[i].title = 'Info';
              break;
            case 'warning':
              this.messages[i].type = 'warning';
              this.messages[i].drupalType = 'warning';
              this.messages[i].icon = this.iconWarning;
              this.messages[i].title = 'Warning';
              break;
            case 'error':
            case 'danger':
              this.messages[i].type = 'danger';
              this.messages[i].drupalType = 'error';
              this.messages[i].icon = this.iconError;
              this.messages[i].title = 'Error';
              break;
            default:
              this.messages[i].type = this.defaultMessageType;
              this.messages[i].icon = this.iconInfo;
              this.messages[i].title = this.defaultMessageTitle;
          }
        }
        this.render();
      }
    },
    render: function () {
      let toastContainer = document.querySelector('#toast-container');
      if (!toastContainer) {
        toastContainer = document.createElement('div');
        toastContainer.classList.add('toast-container', this.position, 'p-3');
        toastContainer.id = 'toast-container';
        toastContainer.setAttribute('role', 'alert');
        toastContainer.setAttribute('aria-live', 'polite');
        toastContainer.setAttribute('aria-atomic', 'true');
        $('body').prepend(toastContainer);
      }
      for (let i in this.messages) {
        toastContainer.appendChild(this.renderItem(this.messages[i]));
      }
      const toastElements = $('.toast');
      switch (this.bootstrapVersion) {
        case 4:
          toastElements.toast('show');
          break;
        case 5:
          const toastElementsArray = Array.from(toastElements);
          toastElementsArray.forEach(function (toastElement) {
            new bootstrap.Toast(toastElement).show();
          });
      }
      toastElements.off('hide.bs.toast').on('hide.bs.toast', function () {
        Drupal.bootstrap_toast_messages.onToastHide(this);
      });
    },
    renderItem: function (message) {
      let toast = document.createElement('div');
      toast.classList.add('toast');
      toast.id = 'toast-' + this.count;
      toast.setAttribute('role', 'alert');
      toast.setAttribute('aria-live', 'assertive');
      toast.setAttribute('aria-atomic', 'true');
      //Animation
      toast.setAttribute(this.dataAttributes.animation, 'false'); //Disable bootstrap animations for custom animations
      if (this.animation.in !== 'none' || this.animation.out !== 'none') {
        toast.classList.add('animate');
        if (this.animation.in !== 'none') {
          toast.classList.add('animate-in', this.animation.in);
        }
        if (this.animation.out !== 'none') {
          toast.classList.add('animate-out', this.animation.out);
        }
        if (this.messages.length > 1) {
          toast.classList.add('animate-in--' + this.count);
        }
      }
      //Bootstrap hide attributes
      if (this.messageTypeDuration[message.drupalType] === 0) {
        toast.setAttribute(this.dataAttributes.autoHide, 'false');
      } else {
        toast.setAttribute(this.dataAttributes.autoHide, 'true');
        toast.setAttribute(this.dataAttributes.delay, this.messageTypeDuration[message.drupalType]);
      }

      let toastHeader = document.createElement('div');
      toastHeader.classList.add('toast-header', 'bg-' + message.type);
      toast.appendChild(toastHeader);

      let toastBody = document.createElement('div');
      toastBody.classList.add('toast-body', 'rounded-bottom', 'p-4', 'd-flex', 'align-items-center');
      toastBody.innerHTML = '' +
        '<span class="toast__icon pr-2 pe-2">' + message.icon + '</span>' +
        '<span class="toast__message">' + message.message + '</span>' +
        '<button type="button" class="ml-2 me-2 mb-1 ms-auto ml-auto toast__close" ' + this.dataAttributes.dismiss + '="toast" aria-label="Close"><svg width="9" height="9" viewBox="0 0 44 44" aria-hidden="true" focusable="false"><path d="M0.549989 4.44999L4.44999 0.549988L43.45 39.55L39.55 43.45L0.549989 4.44999Z" /><path d="M39.55 0.549988L43.45 4.44999L4.44999 43.45L0.549988 39.55L39.55 0.549988Z" /></svg></button>';
      toast.appendChild(toastBody);
      this.count++;
      return toast;
    },
    removeToasts: function () {
      let toasts = $('#toast-container').find('.toast');
      if (toasts.length) {
        toasts.each(function (index, toast) {
          if (toast.classList.contains('hide')) {
            toast.remove();
          }
        });
      }
    },
    onToastHide(toast) {
      if (!toast.classList.contains('animate-out')) {
        toast.remove();
      } else {
        toast.classList.remove('animate-in');
        toast.classList.add('removing');
        let animationDuration = getComputedStyle(toast).animationDuration;
        if (!animationDuration) {
          animationDuration = '500ms';
        }
        let timeout = 0;
        // Convert the animation duration to milliseconds
        if (animationDuration.includes('ms')) {
          timeout = parseFloat(animationDuration); // Already in milliseconds
        } else if (animationDuration.includes('s')) {
          timeout = parseFloat(animationDuration) * 1000; // Convert seconds to milliseconds
        }

        // Get the container as jQuery & its dimensions
        const $container = $(toast.parentNode);
        const containerHeight = $container.height();
        const containerOuterHeight = $container.outerHeight(true);

        //Get the toast as jQuery & its dimensions
        const $toast = $(toast);
        const toastOuterHeight = $toast.outerHeight(true);
        const toastHeight = $toast.height();
        const toastMarginBottom = $toast.css('margin-bottom');

        this.totalRemovingHeight += toastOuterHeight;

        let $next = null;
        let $prev = null;
        if (this.position.startsWith('top')) {
          $next = $toast.next();
        } else if (this.position.startsWith('bottom')) {
          $prev = $toast.prev();
        }
        if (($next && $next.length) || ($prev && $prev.length)) {
          $container.height(containerHeight);
        }

        // Set a timeout to remove the element after the animation duration
        setTimeout(() => {
          toast.remove();
          //Top positioned toasts
          if ($next && $next.length) {
            $next.css({ 'margin-top': toastHeight });
            $next.animate({ 'margin-top': 0 }, this.animation.duration, 'swing', function () {
              $container.css('height', '');
              //Not strictly required here but resetting the totalRemovingHeight
              Drupal.bootstrap_toast_messages.totalRemovingHeight = 0;
            });
          }
          //Bottom positioned toasts
          else if ($prev && $prev.length) {
            const removingCount = $('.toast-container .toast.removing').length;
            $prev.css({ 'margin-bottom': toastHeight });
            $prev.animate({ 'margin-bottom': toastMarginBottom }, this.animation.duration, 'swing', function () {
              if(removingCount === 0) {
                $container.animate({ height: (containerOuterHeight - Drupal.bootstrap_toast_messages.totalRemovingHeight) }, Drupal.bootstrap_toast_messages.animation.duration, 'swing', function () {
                  $container.css('height', '');
                  Drupal.bootstrap_toast_messages.totalRemovingHeight = 0;
                });
              }
            });
          }
        }, Number(timeout));

      }
      this.count--;
    },
  };

  /**
   * Registers behaviours
   */
  Drupal.behaviors.bootstrap_toast_messages = {
    attach: function (context, settings) {
      once(Drupal.bootstrap_toast_messages.id, Drupal.bootstrap_toast_messages.context, context).forEach(function (element) {
        Drupal.bootstrap_toast_messages.init();
        if (typeof settings.bootstrap_toast_messages !== 'undefined' && typeof settings.bootstrap_toast_messages.messages_array !== 'undefined') {
          Drupal.bootstrap_toast_messages.createMessages(settings.bootstrap_toast_messages.messages_array);
          settings.bootstrap_toast_messages.messages_array = null;
        }
      });
    },
  };

  /**
   * Create bootstrap toast messages on ajax requests
   */
  Drupal.AjaxCommands.prototype.bootstrapToastMessagesCommand = function (ajax, response, status) {
    Drupal.bootstrap_toast_messages.createMessages(response.messages_array);
  };

}(jQuery, Drupal, once, drupalSettings));
